/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __oplab_SurfaceViewerOp_h__
#define __oplab_SurfaceViewerOp_h__

namespace fe
{

/**************************************************************************//**
	@brief Operator to look at surfaces in an independent window

	@ingroup oplab
*//***************************************************************************/
class FE_DL_EXPORT SurfaceViewerOp:
	public OperatorSurfaceCommon,
	public Initialize<SurfaceViewerOp>
{
	public:

							SurfaceViewerOp(void);
virtual						~SurfaceViewerOp(void);

		void				initialize(void);

							//* As HandlerI
virtual	void				handle(Record& a_rSignal);

		sp<QuickViewerI>	quickViewer(void)		{ return m_spQuickViewerI; }
		sp<DrawableI>		drawable1(void)			{ return m_spDrawable1; }
		sp<DrawableI>		drawable2(void)			{ return m_spDrawable2; }
		sp<DrawableI>		drawable3(void)			{ return m_spDrawable3; }
		sp<DrawableI>		drawable4(void)			{ return m_spDrawable4; }

		DrawMode::DrawStyle	drawStyle(void) const	{ return m_drawStyle; }
		Real				lineWidth(void) const	{ return m_lineWidth; }

	private:

		void				post(Real a_frame);

	class DrawHandler: virtual public HandlerI
	{
		public:
					DrawHandler(void):
						m_frame(0.0),
						m_us(0)												{}

			void	bind(sp<DrawI> a_spDrawI)
					{	m_spDrawI=a_spDrawI; }

			void	bindDrawable1(sp<DrawableI> a_spDrawable1)
					{ m_spDrawable1=a_spDrawable1; }
			void	bindDrawable2(sp<DrawableI> a_spDrawable2)
					{ m_spDrawable2=a_spDrawable2; }
			void	bindDrawable3(sp<DrawableI> a_spDrawable3)
					{ m_spDrawable3=a_spDrawable3; }
			void	bindDrawable4(sp<DrawableI> a_spDrawable4)
					{ m_spDrawable4=a_spDrawable4; }

			void	setFrame(Real a_frame)		{ m_frame=a_frame; }
			void	setMicroseconds(I32 a_us)	{ m_us=a_us; }

	virtual void	handle(Record& a_render);

		private:

			AsViewer		m_asViewer;
			sp<DrawI>		m_spDrawI;
			sp<DrawableI>	m_spDrawable1;
			sp<DrawableI>	m_spDrawable2;
			sp<DrawableI>	m_spDrawable3;
			sp<DrawableI>	m_spDrawable4;
			Real			m_frame;
			I32				m_us;
	};

	class Worker: public Thread::Functor
	{
		public:
							Worker(sp< JobQueue<Real> > a_spJobQueue,
								U32 a_id,String a_stage):
								m_hpJobQueue(a_spJobQueue),
								m_id(a_id),
								m_us(0)										{}

	virtual					~Worker(void)									{}

	virtual	void			operate(void);

			void			setObject(sp<Component> a_spObject)
							{	m_hpSurfaceViewerOp=a_spObject; }

		private:
			hp< JobQueue<Real> >	m_hpJobQueue;
			U32						m_id;
			U32						m_us;

			hp<SurfaceViewerOp>		m_hpSurfaceViewerOp;
	};

		U32							m_jobs;
		sp< JobQueue<Real> >		m_spJobQueue;
		sp< Gang<Worker,Real> >		m_spGang;
		sp<QuickViewerI>			m_spQuickViewerI;
		sp<DrawHandler>				m_spDrawHandler;
		sp<RecorderI>				m_spVideoRecorder;
		sp<DrawableI>				m_spDrawable1;
		sp<DrawableI>				m_spDrawable2;
		sp<DrawableI>				m_spDrawable3;
		sp<DrawableI>				m_spDrawable4;

		DrawMode::DrawStyle			m_drawStyle;
		Real						m_lineWidth;
		Real						m_lastFrame;

	public:

		sp<DrawHandler>		drawHandler(void)	{ return m_spDrawHandler; }
};

} /* namespace */

#endif /* __oplab_SurfaceViewerOp_h__ */
