/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __viewer_QuickViewerI_h__
#define __viewer_QuickViewerI_h__

namespace fe
{

/**************************************************************************//**
	@brief Self-configuring Viewer

	@ingroup viewer
*//***************************************************************************/
class FE_DL_EXPORT QuickViewerI: virtual public ViewerI
{
	public:
						/// Returns the internally-created WindowI
virtual	sp<WindowI>		getWindowI(void)									=0;

						/// Returns the internally-created Scope
virtual	sp<Scope>		getScope(void)										=0;

						/// Returns the center of attention
virtual	SpatialVector	interestPoint(void)									=0;

						/// Returns the secondary point
virtual	SpatialVector	auxilliaryPoint(void)								=0;

						/// Add an extra handler for the core heartbeat
virtual	void			insertBeatHandler(sp<HandlerI> spHandlerI)			=0;

						/// Add an extra handler for raw event signals
virtual	void			insertEventHandler(sp<HandlerI> spEventHandler)		=0;

						/// Specify a handler for render signals
virtual void			insertDrawHandler(sp<HandlerI> spHandlerI)			=0;

						/// @brief Map the viewer to the screen
virtual	void			open(void)											=0;

						/** @brief Run the viewer for a specified number
							of frames

							The window will be opened automatically.

							For unlimited frames, specify zero. */
virtual	void			run(U32 frames)										=0;
};

} /* namespace */

#endif /* __viewer_QuickViewerI_h__ */
